@@ -29,13 +29,28 @@ except:
     __version__ = 'unknown'
 
 
+class ExecuteSqlError(Exception):
+    pass
+
+
 def execute_sql_file(cursor, filename):
     with open(filename) as fh:
         sql = fh.read()
-        sql = sql.strip()
-        if sql:
-            sql = sqlalchemy.text(sql)
-            cursor.execute(sql)
+    sql = sql.strip()
+    if not sql:
+        return
+    try:
+        sql = sqlalchemy.text(sql)
+        cursor.execute(sql)
+    except sqlalchemy.exc.SQLAlchemyError as e:
+        # https://github.com/sqlalchemy/sqlalchemy/blob/9e7c068d669b209713da62da5748579f92d98129/lib/sqlalchemy/exc.py#L699-L709
+        # To provide more detail on the underlying error, but without printing the original SQL.
+        if e.orig:
+            orig = e.orig
+            error_text = "(%s.%s) %s" % (orig.__class__.__module__, orig.__class__.__name__, str(orig))
+        else:
+            error_text = str(e)
+        raise ExecuteSqlError('Error executing SQL file {}: {}'.format(filename, error_text))
 
 
 class Tusker:
@@ -95,7 +110,7 @@ class Tusker:
         with self.createdb('schema') as schema_engine:
             with schema_engine.begin() as schema_cursor:
                 self.log('Creating original schema...')
-                for filename in sorted(glob(self.config.schema.filename)):
+                for filename in sorted(glob(self.config.schema.filename, recursive=True)):
                     self.log('- {}'.format(filename))
                     execute_sql_file(schema_cursor, filename)
             yield schema_engine
@@ -176,7 +191,7 @@ class Tusker:
     def _get_migrations(self):
         migrations = []
         if self.config.migrations.filename:
-            migrations = glob(self.config.migrations.filename)
+            migrations = glob(self.config.migrations.filename, recursive=True)
         else:
             migrations = [
                 os.path.join(self.config.migrations.directory, filename)
@@ -191,8 +206,12 @@ def cmd_diff(args, cfg: Config):
     target = args.target
     if args.reverse:
         source, target = target, source
-    sql = tusker.diff(source, target)
-    print(sql, end='')
+    try:
+        sql = tusker.diff(source, target)
+        print(sql, end='')
+    except ExecuteSqlError as e:
+        print(str(e), file=sys.stderr)
+        sys.exit(1)
 
 
 def cmd_check(args, cfg: Config):
@@ -200,7 +219,11 @@ def cmd_check(args, cfg: Config):
     if 'all' in backends:
         backends = ['migrations', 'schema', 'database']
     tusker = Tusker(cfg, args.verbose)
-    diff = tusker.check(backends)
+    try:
+        diff = tusker.check(backends)
+    except ExecuteSqlError as e:
+        print(str(e), file=sys.stderr)
+        sys.exit(1)
     if diff:
         print('Schemas differ: {} != {}'.format(diff[0], diff[1]))
         print('Run `tusker diff` to see the differences')
